@@ -5,18 +5,24 @@ use Exporter;
 use Test::More;
 use Text::Diff;
 use FileHandle::Unget;
-use File::Path;
-use File::Spec::Functions qw(:ALL);
+use File::Path qw( remove_tree );
+use File::Spec::Functions qw( :ALL );
+use File::Temp;
+use File::Slurp;
 
 use vars qw( @EXPORT @ISA );
 use Mail::Mbox::MessageParser;
 use Mail::Mbox::MessageParser::MetaInfo;
 
 @ISA = qw( Exporter );
-@EXPORT = qw( CheckDiffs InitializeCache ModuleInstalled
-  Broken_Pipe No_such_file_or_directory
+@EXPORT = qw( CheckDiffs InitializeCache Module_Installed
+  Broken_Pipe No_such_file_or_directory $TEMPDIR
 );
 
+use vars qw( $TEMPDIR );
+
+$TEMPDIR = File::Temp::tempdir();
+
 # ---------------------------------------------------------------------------
 
 sub CheckDiffs
@@ -39,11 +45,8 @@ sub CheckDiffs
 
     if ($numdiffs != 0)
     {
-      open DIFF_OUTPUT, ">$output_filename.diff";
-      print DIFF_OUTPUT "diff \"$output_filename\" \"$filename\"\n";
-      print DIFF_OUTPUT @diffs;
-      close DIFF_OUTPUT;
-
+      write_file("$output_filename.diff", "diff \"$output_filename\" \"$filename\"\n", @diffs);
+    
       print "Failed, with $numdiffs differences.\n";
       print "  See $output_filename.diff.\n";
       ok(0,"Computing differences between $filename and $output_filename");
@@ -66,9 +69,9 @@ sub InitializeCache
 {
   my $filename = shift;
 
-  my $cache_file = catfile('t','temp','cache');
+  my $cache_file = catfile($TEMPDIR,'cache');
 
-  rmtree $cache_file;
+  remove_tree($cache_file) if -e $cache_file;
 
   Mail::Mbox::MessageParser::SETUP_CACHE({'file_name' => $cache_file});
 
@@ -109,7 +112,7 @@ sub InitializeCache
 
 # ---------------------------------------------------------------------------
 
-sub ModuleInstalled
+sub Module_Installed
 {
   my $module_name = shift;
 
@@ -150,13 +153,10 @@ sub No_such_file_or_directory
 # doing this?
 sub Broken_Pipe
 {
-  mkdir catdir('t','temp'), 0700;
-
-  my $script_path = catfile('t','temp','broken_pipe.pl');
+  my $script_path = catfile($TEMPDIR,'broken_pipe.pl');
   my $dev_null = devnull();
 
-  open F, ">$script_path";
-  print F<<EOF;
+  write_file($script_path, <<EOF);
 unless (open B, '-|')
 {
   open(F, "|$^X -pe 'print' 2>$dev_null");
@@ -165,7 +165,6 @@ unless (open B, '-|')
   exit;
 }
 EOF
-  close F;
 
   my $result = `$^X $script_path 2>&1 1>$dev_null`;
 
